

'**********************************************
'Progetto: DDS
'CPU: 18F2550 @ 32MHz (4x8)
'Modulo MAIN
'Versione 10.2 - 31/10/11
'Hardware: DDS_18F2550_XTAL ver. 4 (mod V.5)
'*********************************************
'INTOSC_8MHz_RA6-7 ENABLED
'PLL ENABLED - CLOCK = 32MHz
'*********************************************

'Define SIMULATION_WAITMS_VALUE = 0

Define CONFIG1L = 0x00
Define CONFIG1H = 0x09
Define CONFIG2L = 0x1e
Define CONFIG2H = 0x00
Define CONFIG3L = 0x00
Define CONFIG3H = 0x00
Define CONFIG4L = 0x80
Define CONFIG4H = 0x00
Define CONFIG5L = 0x0f
Define CONFIG5H = 0xc0
Define CONFIG6L = 0x0f
Define CONFIG6H = 0xe0
Define CONFIG7L = 0x0f
Define CONFIG7H = 0x40

Include "mod_DISPLAY.bas"
Include "mod_KEYBOARD.bas"
Include "mod_SPI.bas"
Include "mod_FUNCTIONS.bas"
Include "mod_SERIAL.bas"


'***************** GESTIONE AD9833 *****************
'i bit riservati (non elencati) devono essere a 0
'***************************************************

'Control byte 1 - MSB = ctrh
Const d15 = 7  '15
Const d14 = 6  '14
Const b28 = 5  '13
Const hlb = 4
Const fsel = 3  '11
Const psel = 2  '10
Const rst = 0  '8

'Control byte 0 - LSB = ctrl
Const slp1 = 7
Const slp12 = 6
Const orbiten = 5
Const div2 = 3
Const mode = 1

Const msglen = 14
Const val_tmrlow = 192
Const val_tmrhigh = 224

'----------- PINs symbols --------
'Nota: RA0..RA5 usati per LCD
Symbol marker = RA7

Symbol pr = RB0
Symbol bcd1 = RB1
Symbol bcd2 = RB2
Symbol bcd4 = RB3
Symbol bcd8 = RB4
Symbol fsync = RB5
Symbol cng = RB6  'PGC
Symbol dir = RB7  'PGD

Symbol aux0 = RC0
Symbol opt = RC1
Symbol aux1 = RC2
Symbol sck = RC3
Symbol rw = RC4  'RWRWG LCD
Symbol sdo = RC5
Symbol txd = RC6
Symbol rxd = RC7

'------ Costanti della funzioni ----------
Const mode_normal = 0
Const mode_sweep_single_up = 1
Const mode_sweep_single_down = 2
Const mode_sweep_up = 3
Const mode_sweep_down = 4
Const mode_sweep_updown = 5
Const mode_burst = 6
Const mode_burst_alt = 7
Const mode_num = 7  'numero di modi di funzionamento (0..X)

Const wave_sin = 0
Const wave_tri = 1
Const wave_sqr = 2
Const wave_num = 2  'numero di forma d'onda (0..X)

'----------- GLOBALS -------------
Dim mult As Byte  'moltiplicatore per calcolare la frequenza
Dim ctrl As Byte  'Word di controllo
Dim ctrh As Byte
Dim wave As Byte  'forma d'onda selezionata
Dim gen_mode As Byte

'---- Variabili per gestione frequenze
Dim frq As Long  'valore della frequenza corrente
Dim frq_low As Long  'frequenza iniziale
Dim frq_hgh As Long  'frequenza finale

'Dim current_frq As Long  'salva il valore di fq per i modi Sweep
Dim freqlw As Word  'Word LSB della frequenza
Dim freqhw As Word  'Word MSB della frequenza
Dim frq_0 As Byte  'Bytes della frequenza
Dim frq_1 As Byte
Dim frq_2 As Byte
Dim frq_3 As Byte

'Variabili per gli intervalli di tempo (in msec)
Dim time_on As Word
Dim time_off As Word

'---- Parametri per la gestione modi
Dim sweep_step_cnt As Byte
Dim sweep_delta_freq As Long
Dim sweep_direction As Bit

Dim burst_step_cnt As Word
Dim burst_low As Bit

'---- Variabili per modulo SERIAL
Dim buf(15) As Byte
Dim bufptr As Byte
Dim rxbyte As Byte
Dim rxerror As Byte
Dim cmd As Byte  'comando ricevuto (keyboard/seriale)

'----------- Generic -------------
Dim version As Byte  'versione programma
Dim sub_version As Byte  'sotto versione
Dim vdate As Long

Dim cntmsec As Byte
Dim tmp As Byte
Dim dummyb As Byte
Dim tmp1 As Byte
Dim tmp2 As Byte

'------------ Flags --------------
Dim f_freq_changed As Bit
Dim f_mode_changed As Bit
Dim f_wave_changed As Bit
Dim f_keyb_changed As Bit
Dim f_cmd_changed As Bit

Dim f_inmenu As Bit
Dim f_echo_enabled As Bit
Dim f_serial_enabled As Bit
Dim f_bytereceived As Bit
Dim f_initialized As Bit
Dim f_buffering As Bit
Dim f_triggering As Bit

'----------- altri simboli --------
Symbol wtri = ctrl.1
Symbol orbit = ctrl.5
Symbol dv2 = ctrl.3

'====================================================


OSCCON = %01110000  'INT = 8MHz
OSCTUNE = %01000001  '4x PLL enabled = 32MHz

AllDigital
sck = 1
fsync = 1
marker = 0
txd = 1
rxd = 1
TRISA = %00000000  'all output
TRISB = %00011111  'pr,bcd1..4
TRISC = %10010010  'rxd,rw,opt
txd = 1

'serial
BAUDCON = 0x08  'BRG16 on
TXSTA.SYNC = 0
TXSTA.BRGH = 0
SPBRG = 34  '57600 bps @ FCLK = 32MHz
SPBRGH = 0


'ASM:        clrf baudcon
'ASM:        BSF BAUDCON,BRG16
'ASM:        CLRF TXSTA
'ASM:        BSF TXSTA,BRGH
'ASM:        clrf SPBRGH
'ASM:        MOVLW D  '68' ;67=115,200 bps
'ASM:        MOVWF SPBRG

	
T0CON = %00001000  'tmr off, 16bit, prescaler=bypassed
dummyb = PORTB  'clear port b mismatch

'timer
TMR0H = val_tmrhigh
TMR0L = val_tmrlow  '= 57536  = 65536-8000 : interrupt every 1msec


'interrupts
RCON.IPEN = 1  'abilita high priority interrupt
IPR1.RCIP = 1  'EUSART high priority interrupt
INTCON2 = %00000000  'Pull-up disabled, INT0 rising edge, tmr0 low priority
INTCON3 = %00000000  'not used
INTCON1 = %00000000  'INT0 interrupts disabled, clear flags

ASM:        movlb 0  'Bank 0, per sicurezza

'************ INIZIO PROGRAMMA **************
init:
Lcdinit 0
WaitMs 300

mult = 34  '33.554432
version = 10
sub_version = 2
vdate = 311011

Gosub spi_init
Gosub init_dds  'in FUNCTIONS
Gosub reset_dds

Read 0, dummyb  'version

If dummyb > 0 Then
Read 1, gen_mode
Read 2, wave

Read 3, frq_0
Read 4, frq_1
Read 5, frq_2
Read 6, frq_3
ASM:        movff frq_0,frq_low
ASM:        movff frq_1,frq_low+1
ASM:        movff frq_2,frq_low+2
ASM:        movff frq_3,frq_low+3
Read 7, frq_0
Read 8, frq_1
Read 9, frq_2
Read 10, frq_3
ASM:        movff frq_0,frq_hgh
ASM:        movff frq_1,frq_hgh+1
ASM:        movff frq_2,frq_hgh+2
ASM:        movff frq_3,frq_hgh+3

Read 11, frq_0
Read 12, frq_1
Read 13, frq_2
Read 14, frq_3
ASM:        movff frq_0,time_on
ASM:        movff frq_1,time_on+1
ASM:        movff frq_2,time_off
ASM:        movff frq_3,time_off+1
Endif


Gosub hello
Gosub cambia_forma
Gosub cambia_modo
Call init_menu()  'inizializza i valori di default dei campi
Call get_values()  'copia i valori dei campi nel menu

If f_serial_enabled Then PIE1.RCIE = 1  'abilita l'interrupt EUSART

'abilita la tastiera e la seriale
INTCON.INT0IE = 1
RCSTA.SPEN = 1
RCSTA.CREN = 1
Enable High


While f_initialized = False  'attende pressione di un tasto
Wend

Call display(gen_mode, wave)

'abilita il timer
INTCON.TMR0IE = 1
T0CON.TMR0ON = True
Enable Low


mainloop:


If f_bytereceived Then Gosub serial_cmd

If f_cmd_changed Then Gosub serial_execute

If f_wave_changed = True Then
	Gosub cambia_forma
	f_wave_changed = False
Endif

If f_mode_changed = True Then
	Gosub cambia_modo
	f_mode_changed = False
Endif

If f_freq_changed Then
	Disable Low
	Call calcola_frequenza(frq)
	f_freq_changed = False
	Enable Low
Endif


Goto mainloop

End                                               



On High Interrupt  'INT0 & USART
Save System


If PIR1.RCIF = True Then
	PIR1.RCIF = 0
	rxerror = RCSTA
	rxerror = rxerror And 0x06
	rxbyte = RCREG

	If f_serial_enabled = False Then Goto hint_done
	
	If rxerror > 0 Then
		RCSTA.CREN = 0
		rxbyte = 0
		RCSTA.CREN = 1
		Goto hint_done
	Endif

	f_bytereceived = True

Endif


If INTCON.INT0IF Then
	Call keyboard()
	INTCON.INT0IF = False
	Goto hint_done
Endif


hint_done:
dummyb = PORTB
Resume                                            


On Low Interrupt  'timer
Save System

cntmsec = cntmsec + 1
If cntmsec = 0 Then Toggle dir

marker = 0
If f_triggering Then
	marker = 1
	f_triggering = False
Endif


Select Case gen_mode

Case mode_normal
	
Case mode_burst
	If burst_low = False Then
		If burst_step_cnt < time_on Then
			burst_step_cnt = burst_step_cnt + 1
		Else
			burst_step_cnt = 0
			frq = 0
			f_freq_changed = True
			burst_low = True
			ctrh = %00000001  'reset=1
			Gosub spi_send_command
			ctrh = %00100000  'B28=1 + hlb=0 + reset=1
			Gosub spi_send_command
			f_triggering = True
		Endif
	Else
		If burst_step_cnt < time_off Then
			burst_step_cnt = burst_step_cnt + 1
		Else
			burst_step_cnt = 0
			frq = frq_low
			f_freq_changed = True
			burst_low = False
			f_triggering = True
		Endif
	Endif

	
Case mode_burst_alt
	If burst_low = False Then
		If burst_step_cnt < time_on Then
			burst_step_cnt = burst_step_cnt + 1
		Else
			burst_step_cnt = 0
			frq = frq_hgh
			f_freq_changed = True
			burst_low = True
			f_triggering = True
		Endif
	Else
		If burst_step_cnt < time_off Then
			burst_step_cnt = burst_step_cnt + 1
		Else
			burst_step_cnt = 0
			frq = frq_low
			f_freq_changed = True
			burst_low = False
			f_triggering = True
		Endif
	Endif

	
Case mode_sweep_single_up
	If frq <= frq_hgh Then
		frq = frq + sweep_delta_freq
		f_freq_changed = True
	Endif


Case mode_sweep_single_down
	If frq >= frq_low Then
		frq = frq - sweep_delta_freq
		f_freq_changed = True
	Endif


Case mode_sweep_up
	If frq <= frq_hgh Then
		frq = frq + sweep_delta_freq
		f_freq_changed = True
	Else
		frq = frq_low
		f_freq_changed = True
		f_triggering = True
	Endif

Case mode_sweep_down
	If frq >= frq_low Then
		frq = frq - sweep_delta_freq
		f_freq_changed = True
	Else
		frq = frq_hgh
		f_freq_changed = True
		f_triggering = True
	Endif

Case mode_sweep_updown
	If sweep_direction = False Then
		If frq <= frq_hgh Then
			frq = frq + sweep_delta_freq
			f_freq_changed = True
		Else
			sweep_direction = True
			f_triggering = True
		Endif
	Else
		If frq >= frq_low Then
			frq = frq - sweep_delta_freq
			f_freq_changed = True
		Else
			sweep_direction = False
			f_triggering = True
		Endif
	Endif

EndSelect


	
skpmode:
TMR0H = val_tmrhigh
TMR0L = val_tmrlow  '= 57536  = 65536-8000 : interrupt every 1msec
INTCON.TMR0IF = False
Resume                                            

