'*************************************************************************
'DDS - modulo SPI emulato in SW (bitbanged)
'Versione 5 - 14/11/10
'PIC 18F25x0 @ 4 x 8 MHz(internal clock)
'*************************************************************************

'TEORIA:

'*************************************************************************
'Il modulo espone 3 funzioni al nucleo principale:
'spi_init: predispone i valori a riposo delle linee di comunicazione (una sola chiamata all'avvio del programma)
'spi_send_command: invia al DDS il comando contenuto nei byte 'ctrl' e 'ctrh'
'spi_send_freq: invia al DDS il valore di frequenza da generare, secondo questo formato:

'FQ = valore Long
'= 2 Words da 2 bytes = 'frq_low' e 'frq_high'
'= 4 bytes singoli = 'frq_0', 'frq_1, 'frq_2', 'frq_3'
'*************************************************************************

'Come da Datasheet, il valore della frequenza  specificato tramite 2 Words da 14 bits.
'I 2 bits pi significativi - D15 e D14 - di ciascuna Word stabiliscono
'quale Registro Frequenza utilizzare (F0 o F1), come da tabella:
'01 = Registro 0
'10 = Registro 1

'Questo programma usa solo il Registro 0, quindi D15=0, D14=1

'La programmazione della frequenza richiede questi step:
'1. Attivare (a 0) il comando FSYNC
'2. Inviare la Word bassa (bit meno significativi)
'3. Inviare la Word alta (bit pi significativi)
'4. Disabilitare la ricezione del DDS ponendo FSYNC=1
'*************************************************************************


spi_init:
ASM:        bsf fsync
ASM:        bsf sck
ASM:        bcf sdo
ASM:        return
Return                                            


spi_send_command:
	Call spi_send_word(ctrl, ctrh)
Return                                            



'NOTA:  ctrl  settato dal programma principale con la forma d'onda (Wave) voluta
spi_send_freq:

	ctrh = %00100000  'B28=1,HLB=0, Reset=0
	Gosub spi_send_command

	frq_1 = frq_1 And 0x3f
	frq_1 = frq_1 Or 0x40
	frq_3 = frq_3 And 0x3f
	frq_3 = frq_3 Or 0x40

	Call spi_send_word(frq_0, frq_1)
	Call spi_send_word(frq_2, frq_3)

Return                                            



'trasmette una Word: prima byte MSB, poi byte LSB
Proc spi_send_word(spil As Byte, spih As Byte)
Dim spi_byte As Byte
Dim spi_cnt As Byte
Dim saveintstat As Byte

ASM:        movff intcon,saveintstat  'disabilita interrupts
ASM:        movlw 0x3f
ASM:        andwf intcon,f

ASM:        bcf fsync  'abilita l'AD alla ricezione della Word

'spi_byte = spih
ASM:        movf spih,w
ASM:        movwf spi_byte
ASM:        call send_byte

'spi_byte = spil
ASM:        movf spil,w
ASM:        movwf spi_byte
ASM:        call send_byte

ASM:        bsf fsync  'disabilita la ricezione
ASM:        movff saveintstat,intcon  'riabilita interrupts
ASM:        return



'trasmette un byte tramite SPI (MSBit first)
ASM:send_byte: movlw 8
ASM:        movwf spi_cnt

ASM:ss1:    rlcf spi_byte,f
ASM:        bc ss2
ASM:        bcf sdo
ASM:        bnc ss3

ASM:ss2:    nop
ASM:        bsf sdo

ASM:ss3:    nop
ASM:        bcf sck
ASM:        nop
ASM:        bsf sck
ASM:        decfsz spi_cnt,f
ASM:        goto ss1

ASM:        bcf sdo

ASM:        return

End Proc                                          